# Generated by roxygen2: do not edit by hand

S3method(apply_right,relop)
S3method(as.character,relop)
S3method(column_names,data.frame)
S3method(column_names,relop)
S3method(column_names,relop_drop_columns)
S3method(column_names,relop_extend)
S3method(column_names,relop_non_sql)
S3method(column_names,relop_null_replace)
S3method(column_names,relop_project)
S3method(column_names,relop_rename_columns)
S3method(column_names,relop_select_columns)
S3method(column_names,relop_set_indicator)
S3method(column_names,relop_sql)
S3method(column_names,relop_table_source)
S3method(column_names,relop_theta_join)
S3method(column_names,relop_unionall)
S3method(columns_used,relop)
S3method(columns_used,relop_drop_columns)
S3method(columns_used,relop_extend)
S3method(columns_used,relop_natural_join)
S3method(columns_used,relop_non_sql)
S3method(columns_used,relop_null_replace)
S3method(columns_used,relop_order_expr)
S3method(columns_used,relop_orderby)
S3method(columns_used,relop_project)
S3method(columns_used,relop_rename_columns)
S3method(columns_used,relop_select_columns)
S3method(columns_used,relop_select_rows)
S3method(columns_used,relop_set_indicator)
S3method(columns_used,relop_sql)
S3method(columns_used,relop_table_source)
S3method(columns_used,relop_theta_join)
S3method(columns_used,relop_unionall)
S3method(dim,relop)
S3method(dimnames,relop)
S3method(drop_columns,data.frame)
S3method(drop_columns,relop)
S3method(drop_columns,wrapped_relop)
S3method(ex,wrapped_relop)
S3method(extend,data.frame)
S3method(extend,relop)
S3method(extend,wrapped_relop)
S3method(extend_se,data.frame)
S3method(extend_se,relop)
S3method(extend_se,wrapped_relop)
S3method(format,pre_sql_sub_expr)
S3method(format,pre_sql_token)
S3method(format,relop)
S3method(format,relop_arrow)
S3method(format,relop_natural_join)
S3method(format,relop_theta_join)
S3method(format,relop_unionall)
S3method(format,rquery_db_info)
S3method(format,rquery_non_sql_step)
S3method(format,wrapped_relop)
S3method(format_node,relop_drop_columns)
S3method(format_node,relop_extend)
S3method(format_node,relop_natural_join)
S3method(format_node,relop_non_sql)
S3method(format_node,relop_null_replace)
S3method(format_node,relop_order_expr)
S3method(format_node,relop_orderby)
S3method(format_node,relop_project)
S3method(format_node,relop_rename_columns)
S3method(format_node,relop_select_columns)
S3method(format_node,relop_select_rows)
S3method(format_node,relop_set_indicator)
S3method(format_node,relop_sql)
S3method(format_node,relop_table_source)
S3method(format_node,relop_theta_join)
S3method(format_node,relop_unionall)
S3method(local_td,character)
S3method(local_td,data.frame)
S3method(local_td,name)
S3method(natural_join,data.frame)
S3method(natural_join,relop)
S3method(natural_join,wrapped_relop)
S3method(non_sql_node,data.frame)
S3method(non_sql_node,relop)
S3method(null_replace,data.frame)
S3method(null_replace,relop)
S3method(order_expr,data.frame)
S3method(order_expr,relop)
S3method(order_expr_se,data.frame)
S3method(order_expr_se,relop)
S3method(order_rows,wrapped_relop)
S3method(orderby,data.frame)
S3method(orderby,relop)
S3method(pre_sql_to_query,pre_sql_sub_expr)
S3method(pre_sql_to_query,pre_sql_token)
S3method(print,pre_sql_sub_expr)
S3method(print,pre_sql_token)
S3method(print,relop)
S3method(print,relop_arrow)
S3method(print,rquery_db_info)
S3method(print,rquery_non_sql_step)
S3method(print,wrapped_relop)
S3method(project,data.frame)
S3method(project,relop)
S3method(project,wrapped_relop)
S3method(project_se,data.frame)
S3method(project_se,relop)
S3method(project_se,wrapped_relop)
S3method(rename_columns,data.frame)
S3method(rename_columns,relop)
S3method(rename_columns,wrapped_relop)
S3method(select_columns,data.frame)
S3method(select_columns,relop)
S3method(select_columns,wrapped_relop)
S3method(select_rows,data.frame)
S3method(select_rows,relop)
S3method(select_rows,wrapped_relop)
S3method(select_rows_se,data.frame)
S3method(select_rows_se,relop)
S3method(select_rows_se,wrapped_relop)
S3method(set_indicator,data.frame)
S3method(set_indicator,relop)
S3method(sql_node,data.frame)
S3method(sql_node,relop)
S3method(summary,relop)
S3method(tables_used,relop)
S3method(tables_used,relop_table_source)
S3method(theta_join,data.frame)
S3method(theta_join,relop)
S3method(theta_join_se,data.frame)
S3method(theta_join_se,relop)
S3method(to_sql,relop_drop_columns)
S3method(to_sql,relop_extend)
S3method(to_sql,relop_natural_join)
S3method(to_sql,relop_non_sql)
S3method(to_sql,relop_null_replace)
S3method(to_sql,relop_order_expr)
S3method(to_sql,relop_orderby)
S3method(to_sql,relop_project)
S3method(to_sql,relop_rename_columns)
S3method(to_sql,relop_select_columns)
S3method(to_sql,relop_select_rows)
S3method(to_sql,relop_set_indicator)
S3method(to_sql,relop_sql)
S3method(to_sql,relop_table_source)
S3method(to_sql,relop_theta_join)
S3method(to_sql,relop_unionall)
S3method(wrap,data.frame)
export(actualize_join_plan)
export(affine_transform)
export(aggregate_nse)
export(aggregate_se)
export(arrow)
export(assign_slice)
export(build_join_plan)
export(column_names)
export(columns_used)
export(commencify)
export(complete_design)
export(convert_yaml_to_pipeline)
export(count_null_cols)
export(db_td)
export(dbi_table)
export(describe_tables)
export(drop_columns)
export(ex)
export(example_employee_date)
export(execute)
export(expand_grid)
export(extend)
export(extend_nse)
export(extend_se)
export(format_node)
export(getDBOption)
export(graph_join_plan)
export(if_else_block)
export(if_else_op)
export(inspect_join_plan)
export(key_inspector_all_cols)
export(key_inspector_postgresql)
export(key_inspector_sqlite)
export(local_td)
export(lookup_by_column)
export(make_assignments)
export(map_column_values)
export(mark_null_cols)
export(materialize)
export(materialize_node)
export(materialize_sql)
export(mk_td)
export(natural_join)
export(non_sql_node)
export(normalize_cols)
export(null_replace)
export(op_diagram)
export(order_expr)
export(order_expr_nse)
export(order_expr_se)
export(order_rows)
export(orderby)
export(pick_top_k)
export(pre_sql_fn)
export(pre_sql_identifier)
export(pre_sql_string)
export(pre_sql_sub_expr)
export(pre_sql_to_query)
export(pre_sql_token)
export(project)
export(project_nse)
export(project_se)
export(quantile_cols)
export(quantile_node)
export(quote_identifier)
export(quote_literal)
export(quote_string)
export(quote_table_name)
export(rename_columns)
export(rlook)
export(row_counts)
export(rq_colnames)
export(rq_coltypes)
export(rq_connection_advice)
export(rq_connection_name)
export(rq_connection_tests)
export(rq_copy_to)
export(rq_execute)
export(rq_function_mappings)
export(rq_get_query)
export(rq_head)
export(rq_nrow)
export(rq_remove_table)
export(rq_table_exists)
export(rquery_apply_to_data_frame)
export(rquery_db_info)
export(rquery_default_db_info)
export(rquery_default_methods)
export(rstr)
export(rsummary)
export(rsummary_node)
export(select_columns)
export(select_rows)
export(select_rows_nse)
export(select_rows_se)
export(setDBOpt)
export(setDBOption)
export(set_indicator)
export(sql_expr_set)
export(sql_node)
export(str_pre_sql_sub_expr)
export(table_source)
export(tables_used)
export(theta_join)
export(theta_join_nse)
export(theta_join_se)
export(to_sql)
export(to_transport_representation)
export(tokenize_for_SQL)
export(topo_sort_tables)
export(unionall)
export(wrap)
exportMethods(apply_right_S4)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setMethod)
importFrom(methods,show)
importFrom(methods,signature)
importFrom(stats,sd)
importFrom(wrapr,"%.>%")
importFrom(wrapr,"%:=%")
importFrom(wrapr,apply_left)
importFrom(wrapr,apply_right)
importFrom(wrapr,mk_tmp_name_source)
importMethodsFrom(wrapr,apply_right_S4)
